@@ -6,7 +6,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2022 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2023 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -43,7 +43,7 @@
 #define __attribute__(x) /**/
 #endif
 #define GO_UNUSED __attribute__((unused))
-#define GO_VERSION 		"1.7"
+#define GO_VERSION 		"1.7.1"
 #define GO_WEBSITE 		"https://goaccess.io/"
 extern struct tm now_tm;
 
@@ -115,6 +115,32 @@ typedef enum MODULES {
   TLS_TYPE,
 } GModule;
 
+/* Total number of storage metrics (GSMetric) */
+#define GSMTRC_TOTAL 19
+
+/* Enumerated Storage Metrics */
+typedef enum GSMetric_ {
+  MTRC_KEYMAP,
+  MTRC_ROOTMAP,
+  MTRC_DATAMAP,
+  MTRC_UNIQMAP,
+  MTRC_ROOT,
+  MTRC_HITS,
+  MTRC_VISITORS,
+  MTRC_BW,
+  MTRC_CUMTS,
+  MTRC_MAXTS,
+  MTRC_METHODS,
+  MTRC_PROTOCOLS,
+  MTRC_AGENTS,
+  MTRC_METADATA,
+  MTRC_UNIQUE_KEYS,
+  MTRC_AGENT_KEYS,
+  MTRC_AGENT_VALS,
+  MTRC_CNT_VALID,
+  MTRC_CNT_BW,
+} GSMetric;
+
 /* Metric totals. These are metrics that have a percent value and are
  * calculated values. */
 typedef struct GPercTotals_ {
