@@ -7,7 +7,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2022 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2023 Gerardo Orellana <hello @ goaccess.io>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -109,6 +109,75 @@ reset_struct (Logs * logs) {
     logs->glog[i].invalid = logs->glog[i].processed = 0;
 }
 
+/* Allocate memory for a new Logs and GLog instance.
+ *
+ * On success, the newly allocated Logs is returned . */
+Logs *
+new_logs (int size) {
+  Logs *logs = xmalloc (sizeof (*logs));
+  memset (logs, 0, sizeof *logs);
+
+  logs->glog = xcalloc (size, sizeof (GLog));
+  logs->size = size;
+  logs->idx = 0;
+
+  return logs;
+}
+
+/* Allocate, initialize and add the given filename to our logs structure.
+ *
+ * On error, 1 is returned.
+ * On success, the given filename is added to the Logs structure and 0 is
+ * returned. */
+int
+set_glog (Logs * logs, const char *filename) {
+  GLog *tmp = NULL, *glog = NULL;
+  int newlen = 0;
+  char const *err;
+  char *fvh = NULL, *fn = (char *) filename;
+
+  if (logs->size - 1 < logs->idx) {
+    newlen = logs->size + 1;
+    if (!(tmp = xrealloc (logs->glog, newlen * sizeof (GLog))))
+      return ERR_LOG_REALLOC_FAILURE;
+
+    logs->glog = tmp;
+
+    memset (logs->glog + logs->idx, 0, (logs->idx + 1 - logs->size) * sizeof *logs->glog);
+    logs->size = newlen;
+  }
+
+  glog = logs->glog;
+  glog[logs->idx].errors = xcalloc (MAX_LOG_ERRORS, sizeof (char *));
+  glog[logs->idx].filename = xstrdup (fn);
+  glog[logs->idx].fname = xstrdup (basename (fn));
+
+  if (!glog->pipe && conf.fname_as_vhost) {
+    if (!(fvh = regex_extract_string (glog[logs->idx].fname, conf.fname_as_vhost, 1, &err)))
+      FATAL ("%s %s[%s]", err, glog[logs->idx].fname, conf.fname_as_vhost);
+    glog[logs->idx].fname_as_vhost = fvh;
+  }
+
+  logs->processed = &(glog[logs->idx].processed);
+  logs->filename = glog[logs->idx].filename;
+  logs->idx++;
+
+  return 0;
+}
+
+/* Ensure the given filename is part of our original list of files.
+ *
+ * On error, 1 is returned.
+ * On success, the given filename is added to the Logs structure and 0 is
+ * returned. */
+int
+set_log (Logs * logs, const char *value) {
+  if (str_inarray (value, conf.filenames, conf.filenames_idx) < 0)
+    return ERR_LOG_NOT_FOUND;
+
+  return set_glog (logs, value);
+}
+
 /* Allocate memory for a new set of Logs including a GLog instance.
  *
  * On success, the newly allocated Logs is returned . */
@@ -116,9 +185,7 @@ Logs *
 init_logs (int size) {
   Logs *logs = NULL;
   GLog *glog = NULL;
-  char *fvh = NULL;
-  char const *err;
-  int i = 0;
+  int i = 0, ret = 0;
 
   /* if no logs no a pipe nor restoring, nothing to do then */
   if (!size && !conf.restore)
@@ -126,32 +193,21 @@ init_logs (int size) {
 
   /* If no logs nor a pipe but restoring, we still need an minimal instance of
    * logs and a glog */
-  logs = xcalloc (1, sizeof (*logs));
   if (!size) {
+    logs = xcalloc (1, sizeof (*logs));
     logs->glog = xcalloc (1, sizeof (*glog));
     logs->processed = &(logs->glog[0].processed);
     return logs;
   }
 
-  glog = xcalloc (size, sizeof (*glog));
-  for (i = 0; i < size; ++i) {
-    glog[i].errors = xcalloc (MAX_LOG_ERRORS, sizeof (char *));
-    glog[i].filename = xstrdup (conf.filenames[i]);
-    glog[i].fname = xstrdup (basename (glog[i].filename));
-
-    if (!glog->pipe && conf.fname_as_vhost) {
-      if (!(fvh = regex_extract_string (glog[i].fname, conf.fname_as_vhost, 1, &err)))
-        FATAL ("%s %s[%s]", err, glog[i].fname, conf.fname_as_vhost);
-      glog[i].fname_as_vhost = fvh;
-    }
-
-    logs->processed = &(glog[i].processed);
-    logs->filename = glog[i].filename;
-  }
-
-  logs->glog = glog;
+  logs = new_logs (size);
   logs->size = size;
 
+  for (i = 0; i < size; ++i) {
+    if ((ret = set_log (logs, conf.filenames[i])))
+      FATAL ("%s\n", ERR_LOG_NOT_FOUND_MSG);
+  }
+
   return logs;
 }
 
@@ -784,22 +840,22 @@ spec_err (GLogItem * logitem, int code, const char spec, const char *tkn) {
   const char *fmt = NULL;
 
   switch (code) {
-  case SPEC_TOKN_NUL:
+  case ERR_SPEC_TOKN_NUL:
     fmt = "Token for '%%%c' specifier is NULL.";
     err = xmalloc (snprintf (NULL, 0, fmt, spec) + 1);
     sprintf (err, fmt, spec);
     break;
-  case SPEC_TOKN_INV:
+  case ERR_SPEC_TOKN_INV:
     fmt = "Token '%s' doesn't match specifier '%%%c'";
     err = xmalloc (snprintf (NULL, 0, fmt, (tkn ? tkn : "-"), spec) + 1);
     sprintf (err, fmt, (tkn ? tkn : "-"), spec);
     break;
-  case SPEC_SFMT_MIS:
+  case ERR_SPEC_SFMT_MIS:
     fmt = "Missing braces '%s' and ignore chars for specifier '%%%c'";
     err = xmalloc (snprintf (NULL, 0, fmt, (tkn ? tkn : "-"), spec) + 1);
     sprintf (err, fmt, (tkn ? tkn : "-"), spec);
     break;
-  case SPEC_LINE_INV:
+  case ERR_SPEC_LINE_INV:
     fmt = "Incompatible format due to early parsed line ending '\\0'.";
     err = xmalloc (snprintf (NULL, 0, fmt, (tkn ? tkn : "-")) + 1);
     sprintf (err, fmt, (tkn ? tkn : "-"));
@@ -877,10 +933,10 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       dspc = find_alpha_count (pch);
 
     if (!(tkn = parse_string (&(*str), end, MAX (dspc, fmtspcs) + 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     if (str_to_time (tkn, dfmt, &tm, 1) != 0 || set_date (&logitem->date, tm) != 0) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -894,10 +950,10 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->time)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     if (str_to_time (tkn, tfmt, &tm, 1) != 0 || set_time (&logitem->time, tm) != 0) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -910,11 +966,11 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->time && logitem->date)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     if (str_to_time (tkn, tfmt, &tm, 1) != 0 || set_date (&logitem->date, tm) != 0 ||
         set_time (&logitem->time, tm) != 0) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -929,7 +985,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL)
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
     logitem->vhost = tkn;
     break;
     /* remote user */
@@ -938,7 +994,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL)
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
     logitem->userid = tkn;
     break;
     /* cache status */
@@ -947,7 +1003,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL)
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
     if (is_cache_hit (tkn))
       logitem->cache_status = tkn;
     else
@@ -962,17 +1018,17 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (*str[0] == '[' && (*str += 1) && **str)
       end = "]";
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     if (!conf.no_ip_validation && invalid_ipaddr (tkn, &logitem->type_ip)) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
     /* require a valid host token (e.g., ord38s18-in-f14.1e100.net) even when we're
      * not validating the IP */
     if (conf.no_ip_validation && *tkn == '\0') {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -983,11 +1039,11 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->method)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
     {
       const char *meth = NULL;
       if (!(meth = extract_method (tkn))) {
-        spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+        spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
         free (tkn);
         return 1;
       }
@@ -1002,11 +1058,11 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL || *tkn == '\0') {
       free (tkn);
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
     }
 
     if ((logitem->req = decode_url (tkn)) == NULL) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -1023,7 +1079,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     }
 
     if ((logitem->qstr = decode_url (tkn)) == NULL) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -1034,11 +1090,11 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->protocol)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
     {
       const char *proto = NULL;
       if (!(proto = extract_protocol (tkn))) {
-        spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+        spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
         free (tkn);
         return 1;
       }
@@ -1051,7 +1107,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->req)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     logitem->req = parse_req (tkn, &logitem->method, &logitem->protocol);
     free (tkn);
@@ -1061,7 +1117,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->status)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     /* do not validate HTTP status code */
     if (conf.no_strict_status) {
@@ -1071,7 +1127,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
 
     status = strtol (tkn, &sEnd, 10);
     if (tkn == sEnd || *sEnd != '\0' || errno == ERANGE || status < 100 || status > 599) {
-      spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
+      spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
@@ -1082,7 +1138,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->resp_size)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     bandw = strtoull (tkn, &bEnd, 10);
     if (tkn == bEnd || *bEnd != '\0' || errno == ERANGE)
@@ -1148,7 +1204,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->serve_time)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     serve_secs = strtoull (tkn, &bEnd, 10);
     if (tkn == bEnd || *bEnd != '\0' || errno == ERANGE)
@@ -1166,7 +1222,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->serve_time)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     if (strchr (tkn, '.') != NULL)
       serve_secs = strtod (tkn, &bEnd);
@@ -1187,7 +1243,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->serve_time)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     serve_time = strtoull (tkn, &bEnd, 10);
     if (tkn == bEnd || *bEnd != '\0' || errno == ERANGE)
@@ -1197,14 +1253,31 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     contains_usecs ();  /* set flag */
     free (tkn);
     break;
+    /* time taken to serve the request, in nanoseconds */
+  case 'n':
+    /* ignore it if we already have served time */
+    if (logitem->serve_time)
+      return 0;
+    if (!(tkn = parse_string (&(*str), end, 1)))
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
+    serve_time = strtoull (tkn, &bEnd, 10);
+    if (tkn == bEnd || *bEnd != '\0' || errno == ERANGE)
+      serve_time = 0;
+
+    /* convert it to microseconds */
+    logitem->serve_time = (serve_time > 0) ? serve_time / MILS : 0;
+
+    contains_usecs ();  /* set flag */
+    free (tkn);
+    break;
     /* UMS: Krypto (TLS) "ECDHE-RSA-AES128-GCM-SHA256" */
   case 'k':
     /* error to set this twice */
     if (logitem->tls_cypher)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
 #if defined(HAVE_LIBSSL) && defined(HAVE_CIPHER_STD_NAME)
     {
@@ -1227,7 +1300,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->tls_type)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     logitem->tls_type = tkn;
     break;
@@ -1238,7 +1311,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (logitem->mime_type)
       return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
-      return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
     logitem->mime_type = tkn;
 
@@ -1358,7 +1431,7 @@ find_xff_host (GLogItem * logitem, char **str, char **p) {
   int res = 0;
 
   if (!(skips = extract_braces (p)))
-    return spec_err (logitem, SPEC_SFMT_MIS, **p, "{}");
+    return spec_err (logitem, ERR_SPEC_SFMT_MIS, **p, "{}");
 
   /* if the log format current char is not within the braces special chars, then
    * we assume the range of IPs are within hard delimiters */
@@ -1391,7 +1464,7 @@ special_specifier (GLogItem * logitem, char **str, char **p) {
     /* XFF remote hostname (IP only) */
   case 'h':
     if (find_xff_host (logitem, str, p))
-      return spec_err (logitem, SPEC_TOKN_NUL, 'h', NULL);
+      return spec_err (logitem, ERR_SPEC_TOKN_NUL, 'h', NULL);
     break;
   }
 
@@ -1423,7 +1496,7 @@ parse_format (GLogItem * logitem, char *str, char *lfmt) {
       continue;
     }
     if (*str == '\0')
-      return spec_err (logitem, SPEC_LINE_INV, '-', NULL);
+      return spec_err (logitem, ERR_SPEC_LINE_INV, '-', NULL);
     if (*str == '\n')
       return 0;
 
